@@ -2,7 +2,7 @@
 
 This module contains data input and output utilities.
 
-Copyright 2015-2022 Lukas Kontenis
+Copyright 2015-2023 Lukas Kontenis
 Contact: dse.ssd@gmail.com
 """
 import os
@@ -252,7 +252,7 @@ def read_bin_file(file_name):
 
 
 def list_files_with_extension(
-        path=None, ext="dat",
+        path=None, ext="dat", recursive_dir_search=False,
         name_exclude_filter=None, name_include_filter=None):
     """List files that have a specific extension."""
 
@@ -270,7 +270,12 @@ def list_files_with_extension(
     if path is None:
         path = '.\\'
 
-    List = os.listdir(path)
+    if recursive_dir_search:
+        List = []
+        for root, subdirs, files in os.walk(path):
+            List += [Path(root) / Path(file_name) for file_name in files]
+    else:
+        List = [Path(entry) for entry in os.listdir(path)]
 
     Paths = []
 
@@ -279,23 +284,21 @@ def list_files_with_extension(
         if name_exclude_filter:
             if isarray(name_exclude_filter):
                 for name_exclude_filter1 in name_exclude_filter:
-                    if(FileName.find(name_exclude_filter1) != -1):
+                    if(str(FileName).find(name_exclude_filter1) != -1):
                         filter_hit = True
                         break
             else:
-                if(FileName.find(name_exclude_filter1) != -1):
+                if(str(FileName).find(name_exclude_filter1) != -1):
                     filter_hit = True
                     break
 
         if name_include_filter:
-            if(FileName.find(name_include_filter) == -1):
+            if(str(FileName).find(name_include_filter) == -1):
                 filter_hit = True
                 continue
 
-        if not filter_hit:
-            ext_ind = FileName.rfind(".")
-            if(ext_ind != -1 and FileName[ext_ind+1:] == ext):
-                Paths.append(str(Path(path).joinpath(FileName)))
+        if not filter_hit and FileName.suffix[1:] == ext:
+            Paths.append(str(FileName.absolute()))
 
     return Paths
 
