@@ -3,7 +3,7 @@
 This module contains plotting utilities. Many of these routines can probably be
 replaced with better alternatives or even internal Python functions.
 
-Copyright 2015-2022 Lukas Kontenis
+Copyright 2015-2023 Lukas Kontenis
 Contact: dse.ssd@gmail.com
 """
 import copy
@@ -82,6 +82,13 @@ def get_def_fig_size(numr, numc):
 
     return [fig_w/25.4, fig_h/25.4]
 
+def get_figure_size(fig_width='1.5x'):
+    if fig_width == '1x':
+        return [3.504, 2.635]
+    elif fig_width == '1.5x':
+        return [4.724, 2.635]
+    elif fig_width == '2x':
+        return [7.205, 2.635]
 
 def figure_with_subplots(numr=2, numc=2, title=None):
     """Create a figure with a subplot grid."""
@@ -177,27 +184,35 @@ def show_png_ext(FileName):
 def hist(
         yarr, yrng=None, target_num_bins=256, hist_bin_sz_fac=1, ygran=None,
         with_histogram_dither=False, hist_dith_factor=4, hide_x_labels=False,
-        orientation="vertical", xmarkers=None, ymarkers=None, ylim=None, log=False,
+        orientation="vertical", xmarkers=None, ymarkers=None, ylim=None, log=False, legend_str=None,
         color=matplotlib.cm.get_cmap('Blues')(0.95)):
     """Plot a nice histogram with dithering and data granularity awareness."""
     if yrng is None:
         yrng = [np.min(yarr), np.max(yarr)]
 
     yspan = yrng[1] - yrng[0]
+
+    if not isinstance(yarr[0], (float, int)):
+        yarr_all = np.concatenate([arr for arr in yarr])
+    else:
+        yarr_all = yarr
+        yarr = [yarr]
+
     if ygran is None:
-        ygran = get_granularity(yarr)
-    ymean = np.nanmean(yarr)
+        ygran = get_granularity(yarr_all)
+
+    ymean = np.nanmean(yarr_all)
 
     print('Y granularity is {:.2e}, scale range is {:.2f} bits'.format(
         ygran, np.log2(yspan/ygran)))
 
-    unique_vals = np.unique(yarr).astype('int32')
+    unique_vals = np.unique(yarr_all)
 
     if len(unique_vals) < target_num_bins:
         # If the number of unique data values is less than the number of
         # target histogram bins, the histogram will have bins with no values in
         # them, which indicates that the histogram resolution is too high.
-        bins = hist_bin_centers_to_edges(unique_vals)
+        bins = hist_bin_centers_to_edges(np.unique(yarr_all))
     else:
         num_bins = yspan/ygran
         ygrans_per_bin = np.max([np.round(num_bins/target_num_bins), 1])
@@ -218,16 +233,23 @@ def hist(
         # The strength of the dithering effect can be controlled by changing
         # the hist_dith_fac factor.
         bin_sz = bins[1] - bins[0]
-        dith = np.random.normal(1, bin_sz/ymean*hist_dith_factor, len(yarr))
+        dith = np.random.normal(1, bin_sz/ymean*hist_dith_factor, len(yarr_all))
         print("Applying histogram dithering with a factor of "
               "{:d}. ".format(hist_dith_factor) +
               "This is an experimental feature, so you might want to compare "
               " the histogram with and without dithering.")
+
     else:
         dith = 1
 
-    plt.hist(yarr*dith, bins=bins, orientation=orientation,
+    yarr = [arr*dith for arr in yarr]
+
+    plt.hist(yarr, bins=bins, orientation=orientation, stacked=True,
              color=color, zorder=10, log=log, alpha=0.5)
+
+    if legend_str:
+        plt.legend(legend_str)
+
     ax = plt.gca()
 
     if xmarkers:
@@ -428,6 +450,71 @@ def imshow_ex(
         tick_labels = [get_human_val_str(10**x) for x in ticks]
         cbar_ax.set_xticklabels(tick_labels)
 
+
+def img_with_profiles(
+        img, disp_rng, cmap='viridis', img_title=None,
+        xprof_index=None, yprof_index=None,
+        show_section_avg_profile=False, show_avg_prof_value=False):
+    """Show image with profile panels."""
+    num_px = np.mean(img.shape[:2])
+    if not xprof_index:
+        xprof_index = int(num_px/2)
+    if not yprof_index:
+        yprof_index = int(num_px/2)
+
+    profile_color = get_color('gray')
+
+    plt.imshow(img, vmin=disp_rng[0], vmax=disp_rng[1], cmap=cmap)
+    plt.title(img_title)
+    plt.xticks([])
+    plt.yticks([])
+
+    # Get position coordinates of the image axes
+    pos = plt.gca().get_position()._points
+
+    # Axes creation using plt.axes uses relative figure coordinates, so to make
+    # horizontal and vertical profile views of the same size the figure aspect
+    # ratio needs to be taken into account
+    profile_axes_sz = (pos[1, 0]-pos[0, 0])*0.4
+    fig_ar = plt.gcf().bbox.height/plt.gcf().bbox.width
+
+    # Create new axes, note that get_position() returns corner points, but
+    # axes() expects x0, y0, width, height
+    plt.axes([
+        pos[0, 0],
+        pos[0, 1] - profile_axes_sz,
+        pos[1, 0] - pos[0, 0],
+        profile_axes_sz])
+
+    plt.xticks([])
+    plt.yticks([])
+
+    plt.xlim([0, num_px])
+    plt.ylim(disp_rng)
+    if show_avg_prof_value:
+        add_y_marker(np.nanmean(img[yprof_index-5:yprof_index+5]), ls='-')
+    if show_section_avg_profile:
+        plt.plot(np.arange(0, num_px), np.nanmean(img[yprof_index-5:yprof_index+5, :], 0))
+    plt.plot(np.arange(0, num_px), img[yprof_index, :], c=profile_color)
+
+    plt.axes([
+        pos[1 ,0],
+        pos[0, 1],
+        profile_axes_sz*fig_ar,
+        pos[1, 1] - pos[0, 1]])
+    plt.xticks([])
+    plt.yticks([])
+
+    plt.xlim(disp_rng)
+    plt.ylim([0, num_px])
+    if show_avg_prof_value:
+        add_x_marker(np.nanmean(img[:, xprof_index-5:xprof_index+5]), ls='-')
+    if show_section_avg_profile:
+        plt.plot(np.nanmean(img[:, xprof_index-5:xprof_index+5], 1), np.arange(0, num_px))
+    plt.plot(img[:, xprof_index], np.arange(0, num_px), c=profile_color)
+
+
+
 def add_infobox(
         infobox_str=None, show_infobox=True, infobox_hpos='left', text_va='top',
         infobox_vpos='top', ref_y=None, text_x=None, text_y=None, **kwargs):
