@@ -3,7 +3,7 @@
 This module contains misc utilities. Many of these routines can probably be
 replaced with better alternatives or even internal Python functions.
 
-Copyright 2015-2022 Lukas Kontenis
+Copyright 2015-2023 Lukas Kontenis
 Contact: dse.ssd@gmail.com
 """
 import os
@@ -336,6 +336,9 @@ def get_common_range(*args, mode='overlap', expand_frac=0, symmetric=False):
 
 def get_granularity(A):
     """Get the granularity of a digitally-sampled signal."""
+    if not isinstance(A[0], (float, int)):
+        return np.nanmin([get_granularity(sub_arr) for sub_arr in A])
+
     d = np.abs(np.diff(A))
     d = d[d != 0]
     return np.nanmin(d)
@@ -366,7 +369,7 @@ def decimate(A, bins=2, stride=1):
     return C
 
 
-def cut_by_x_range(X, Y, rng=None, rng_fr=None, rng_to=None):
+def cut_by_x_range(X, Y, rng=None, rng_fr=None, rng_to=None, round_far_range_up=False):
     """Cut X, Y arrays to a range in X.
 
     The X array is assumed to contain monotonously increasing values. The X and
@@ -387,15 +390,19 @@ def cut_by_x_range(X, Y, rng=None, rng_fr=None, rng_to=None):
         print("X and Y need to have equal lengths")
         return [X, Y]
 
+    if rng is not None:
+        rng_fr = rng[0]
+        rng_to = rng[1]
+
+    # Don't do anything if the X array already fits the requested range
+    if np.nanmin(X) > rng_fr and np.nanmax(X) < rng_to:
+        return [X, Y]
+
     # Sort arrays
     sort_order = np.argsort(X)
     X = X[sort_order]
     Y = Y[sort_order]
 
-    if rng is not None:
-        rng_fr = rng[0]
-        rng_to = rng[1]
-
     if(isnone(rng_fr) and isnone(rng_to)):
         return [X, Y]
 
@@ -409,6 +416,9 @@ def cut_by_x_range(X, Y, rng=None, rng_fr=None, rng_to=None):
     else:
         rng_to_ind = find_closest(X, rng_to)
 
+    if round_far_range_up:
+        rng_to_ind += 1
+
     return [X[rng_fr_ind:rng_to_ind], Y[rng_fr_ind:rng_to_ind]]
 
 
@@ -675,7 +685,7 @@ def bin_data_by_x(X, Y, binsz_X):
     return [Xb, Yb, Yb_sd]
 
 
-def bin_data_on_grid(xarr, yarr, xgrid, verbose=False):
+def bin_data_on_grid(xarr, yarr, xgrid, func='avg', verbose=False):
     """Bin data on a grid.
 
     Useful for averaging data while resampling on a nonmonotonous grid, e.g.
@@ -684,6 +694,7 @@ def bin_data_on_grid(xarr, yarr, xgrid, verbose=False):
     ygrid = np.empty_like(xgrid)
     if verbose:
         print("Binning data on grid...")
+        print("Function: {:}".format(func))
     for ind, xval in enumerate(xgrid):
         if verbose and np.mod(ind, 10) == 0:
             print("{:d}/{:d}".format(ind, len(xgrid)))
@@ -696,7 +707,10 @@ def bin_data_on_grid(xarr, yarr, xgrid, verbose=False):
         else:
             ind_to = len(xarr)
 
-        ygrid[ind] = np.nanmean(yarr[ind_from:ind_to])
+        if func == 'max':
+            ygrid[ind] = np.nanmax(yarr[ind_from:ind_to])
+        elif func == 'avg':
+            ygrid[ind] = np.nanmean(yarr[ind_from:ind_to])
 
     return ygrid
 
@@ -749,7 +763,7 @@ def bin_and_slice_data(X=None, Y=None, num_slcs=None, num_bins=None):
     if(sz <= num_bins):
         Xb = X
         Yb = Y
-        Yb_lvls = np.zeros_like(Xb, len(fracs))
+        Yb_lvls = None
         return [Xb, Yb, Yb_lvls]
 
     bin_step = round(sz/num_bins)
@@ -769,7 +783,11 @@ def bin_and_slice_data(X=None, Y=None, num_slcs=None, num_bins=None):
         Xb[ind] = np.mean(X[ind_fr:ind_to])
         Yb[ind] = np.mean(Y[ind_fr:ind_to])
 
-        Yb_lvls[ind, :] = np.percentile(Y[ind_fr:ind_to], fracs)
+        if ind_fr < ind_to:
+            Yb_lvls[ind, :] = np.percentile(Y[ind_fr:ind_to], fracs)
+        else:
+            Yb_lvls[ind, :] = np.nan
+
 
     return [Xb, Yb, Yb_lvls]
 
